#include "io430.h"
#include "font.h"
#include "mmc.h"
//#include "intrinsics.h"

#if   defined  (TLS8204)
#define CONTRAST        0x5A
#elif defined  (PCD8544)
#define CONTRAST        0x7F
#endif

//Global definitions +
#define OSC_FREQ 18000000
#define LCD_FREQ  1000000
#define MMC_FREQ  1000000
#define BAUDRATE     9600
//Global definitions -

//LCD definitions +
#define SEND_CMD 0
#define SEND_CHR 1
#define LCD_X_RES                  84
#define LCD_Y_RES                  48
#define LCD_START_LINE_ADDR	(66-2)
//LCD definitions -

//Optional definitions +
#define HOLD_TIME 50
#define PULL_HOLD_TIME 50
#define TIMEOUT 50
//Optional definitions -

//Extension masks +

#define P1MASK   (char) 0x83      //24; 18-17
#define P2MASK   (char) 0xFF      //32-25
#define P3MASK   (char) 0x0F      //36-33
#define P4MASK   (char) 0xFF      //50-43
#define P5MASK   (char) 0x00      //none
#define P6MASK   (char) 0x7F      //3-1; 100-97
#define P7MASK   (char) 0xFC      //8-5; 56-55
#define P8MASK   (char) 0xFD      //67-65; 61-59; 57
#define P9MASK   (char) 0x0E      //71-69
#define P10MASK  (char) 0x0F      //79-76
#define P11MASK  (char) 0x07      //86-84


#define UMASK  (char) 0xF0        //UExt mask

//Extension masks -



//global variables
unsigned char  LCDMemory[504], a[12][8];
unsigned long index;
char page=1, line=1, inverse=1, released=1, break_f=0, counter, g_f=1, f=1, test_const, MMC_Flag=1;

//declare prototypes of functions +

  //misc functions
  void Delay(unsigned long time);
  void PORT_Init();
  void CLOCK_XT1_Init();
  void CLOCK_XT2_Init();
  void Action(char p, char l);
  void Joystick_Check(char menu);
  void PIN_Init();

  //UART2 functions
  void UART2_Init(unsigned long baud);
  void UART2_Transmit(unsigned char data);
  unsigned char UART2_Receive();

  //Terminal functions
  void Terminal_Write(unsigned char *a);
  void Terminal_WriteLn(unsigned char *a);
  void Terminal_WriteNum(unsigned long a);
  void Terminal_NewLine();

  //UART3 (USB) functions
  void UART3_Init(unsigned long baud);
  unsigned char UART3_Receive();
  void UART3_Transmit (unsigned char data);

  //SPI2 & LCD functions
  void SPI2_Init(unsigned long freq);
  void SPI2_Transmit(unsigned char data);
  unsigned char SPI2_Receive();
  void LCD_Init();
  void LCD_Send(unsigned char data, unsigned char cd);
  void LCD_Clear();
  void LCD_Update(unsigned char mode, unsigned char num);
  void LCD_Contrast(unsigned char contrast);
  void LCD_Clear_Memory();
  void LCD_Set_Position(unsigned char x, unsigned int y);
  void LCD_Char_XY(char x, char y, unsigned char ch, char mode);
  void LCD_NewLine();
  void LCD_Write(unsigned char *a);
  void LCD_WriteLn(unsigned char *a);
  void LCD_WriteNum(unsigned long a);
  void LCD_Menu(char p);
  void LCD_Inverse();

  //ADC12 & TimerA ("DAC") functions
  void ADC12_Init();
  unsigned int ADC12_Receive();
  void TIMER_A_Init();

  //Extension functions
  void Extension_Init();
  unsigned char Extension_Check();
  void Extension_Locate(unsigned char p, unsigned char port);
  void Extension_Locate_PIN(unsigned char port1, unsigned char pin1, unsigned port2, unsigned char pin2);
  void Extension_Locate_PullPin(int x, int y);

  //UExt functions
  void UExt_Init();
  unsigned char UExt_Check();

  //SPI0 & MMC functions
  void SPI0_Init(unsigned long freq);
  void SPI0_Transmit(unsigned char data);
  unsigned char SPI0_Receive();

  //Test functions
  void TEST_LED_1();
  void TEST_LED_2();
  void TEST_ADC();
  void TEST_Extension();
  void TEST_UExt();
  void Test_UART3();

//declare prototypes of functions -



//definitions of functions +

  //misc functions +

    void Delay(unsigned long time)
    {
      while (time--);
    }

    //optional init
    void PIN_Init()
    {
      int i;
      //PORT1 +
      for (i=0; i<=7; i++)
        a[1][i] = 17+i;
      //PORT1 -

      //PORT2 +
      for (i=0; i<=7; i++)
        a[2][i] = 25+i;
      //PORT2 -

      //PORT3 +
      for (i=0; i<=7; i++)
        if (i<=3)
          a[3][i] = 33+i;
        else
          a[3][i] = 35+i;
      //PORT3 -

      //PORT4 +
      for (i=0; i<=7; i++)
        a[4][i] = 43+i;
      //PORT4 -

      //PORT5 +
      a[5][0] = 9; a[5][1] = 10;
      a[5][2] = 89; a[5][3] = 90;
      for (i=4; i<=7; i++)
        a[5][i] = 47+i;
      //PORT5 -

      //PORT6 +
      for (i=0; i<=7; i++)
        if (i<=3)
          a[6][i] = 97+i;
        else
          a[6][i] = -3+i;
      //PORT6 -

      //PORT7 +
      a[7][0] = 13; a[7][1] = 14;
      a[7][2] = 55; a[7][3] = 56;
      for (i=4; i<=7; i++)
        a[7][i] = 1+i;
      //PORT7 -

      //PORT8 +
      for (i=0; i<=7; i++)
        if (i<=4)
          a[8][i] = 57+i;
        else
          a[8][i] = 60+i;
      //PORT8 -

      //PORT9 -
      for (i=0; i<=7; i++)
        a[9][i] = 68+i;
      //PORT9 -

      //PORT10 +
      for (i=0; i<=7; i++)
        a[10][i] = 76+i;
      //PORT10 -

      //PORT11 +
      for (i=0; i<=2; i++)
        a[11][i] = 84+i;
      //PORT11 -
    }

    void PORT_Init()
    {
      //Buttons (Joystick) PORTS
      P1DIR = P1DIR & 0x83;                   //set P1.2/P1.3/P1.4/P1.5/P1.6 (5 buttons as input)

      //LEDs PORTS
      P10OUT = P10OUT & 0x3F;                 //Turn off leds
      P10DIR = P10DIR | 0xC0;                 //set leds as output

      //SPI0 & MMC PORTS
      P3SEL |= 0x0E;				//SIMO, SOMI, CLOCK alternative
      P3OUT |= 0x01;				//Chip Select to High
      P3DIR |= 0x0B;				//Chip Select (MMC), SIMO, CLOCK output

      //XT1 Oscilator PORTS
      P7SEL = P7SEL | 0x03;                   //set XT1 pins as alternate function
      P7DIR = P7DIR | 0x03;                   //set XIN/XOUT as output

      //XT2 Oscilator PORTS
      P5SEL = P5SEL | 0x0C;                   //set XT2 pins as alternate function
      P5DIR = P5DIR | 0x0C;                   //set XIN/XOUT as output

      //Clock PORTS
      P11SEL = P11SEL | 0x07;                 //P11.0/P11.1/P11.2 set as alternate funcions
      P11DIR = P11DIR | 0x07;                 //P11.0/P11.1/P11.2 set as output

      //UART2 PORTS
      P9SEL = P9SEL | 0x30;                   //set P9.4=1 & P9.5=1 as TX/RX
      P9DIR = P9DIR | 0x10;                   //set P9.4=1 (Transmitter) as output
      P9DIR = P9DIR & 0xDF;                   //set P9.5=0 (Receiver) as input

      //SPI & LCD PORTS
      P9SEL = P9SEL | 0x0E;                   //P9.1/P9.2/P9.3 set as alternate function
      P9DIR = P9DIR | 0x0B;                   //P9.0/P9.1/P9.3 set as output
      P9DIR = P9DIR | 0xC0;                   //LCD Reset, LCD data/command bits set as output

      //ADC PORTS
      P8SEL = P8SEL | 0x02;                   //PP8.1 --> ADC (ADC_OUT - remake) function
      P6SEL = P6SEL | 0x80;                   //P6.7 --> ADC function
      P8DIR = P8DIR | 0x02;                   //P8.1 (ADC_OUT - remake) set as output
      P6DIR = P6DIR & 0x7F;                   //PORT6.7 (ADC_IN)  set as input
    }

    void CLOCK_XT1_Init()
    {
      int timeout = 5000;
      P11DIR = P11DIR | 0x01;                   //set P11.0 as output
      P11SEL = P11SEL | 0x01;                   //set P11.0 as ACLK function
      P7SEL = P7SEL | 0x01 | 0x02;              //set P7.0 & P7.1

      UCSCTL6 = UCSCTL6 & (~XT1OFF);            //set XT1OFF=0 (bit0 = 0) --> enable XT1
      UCSCTL3 = UCSCTL3 | 1<<5;                 //set SELREF = 010 --> FLL clock source = REFOCLK
      UCSCTL4 = UCSCTL4 | 1<<9;                 //set SELA = 010 --> ACLK=REFO

      do
      {
        if (!(UCSCTL7 & 0x02)) break;
        UCSCTL7 = UCSCTL7 & ~(0x0F);            // Clear XT2,XT1,DCO fault flags
        SFRIFG1 = SFRIFG1 & ~(0x02);            // Clear fault flags (SFRIFG1.OFIFG = 0)
        timeout--;
      }
      while ((SFRIFG1 & 0x02) && timeout);      // Test oscillator fault flag

      UCSCTL6 = (UCSCTL6 & ~(1<<6)) & ~(1<<7);  //XT1Drive = 00; --> oscilator range 4-8 MHz

      UCSCTL4 = UCSCTL4 & 0xF8FF;               //ACLK=XT1  (SELA = 0b000)
    }


    void CLOCK_XT2_Init()
    {
      int timeout = 5000;
      P11DIR = P11DIR | 0x02 | 0x04;            //set P11.1 & P11.2 as output
      P11SEL = P11SEL | 0x02 | 0x04;            //set P11.1 & P11.2 as MCLK/SMCLK function
      P5SEL = P5SEL | 0x04 | 0x08;              //set P5.2 & P5.3 as XT2IN/XT2OUT

      UCSCTL6 = UCSCTL6 & (~XT2OFF);            //set XT2OFF=0 (bit8 = 0) --> enable XT2
      UCSCTL3 = UCSCTL3 | 0x20;                 //set SELREF = 010 --> FLL clock source = REFOCLK
      UCSCTL4 = UCSCTL4 | 0x200;                //set SELA = 010 --> ACLK=REFO

      do
      {
        if (!(UCSCTL7 & 0x08)) break;
        UCSCTL7 = UCSCTL7 & ~(0x0F);            // Clear XT2,XT1,DCO fault flags
        SFRIFG1 = SFRIFG1 & ~(0x02);            // Clear fault flags (SFRIFG1.OFIFG = 0)
        timeout--;
      }
      while ((SFRIFG1 & 0x02) && timeout);      // Test oscillator fault flag

    //  if (!timeout)
    //    XT2_ERROR = 1;

      UCSCTL5 = UCSCTL5 | 0x0005;               //MCLK/SMCLK divisor = 32

      UCSCTL4 = UCSCTL4 | 0x55;                 //SMCLK=MCLK=XT2
      UCSCTL4 = ((UCSCTL4 & ~0x100) & ~0x200) & ~0x400;     //ACLK=XT1  (SELA = 0b000)
    }

  //misc functions -

  //UART2 & HyperTerminal functions +

    void UART2_Init(unsigned long baud)
    {
      float n, x;
      P9DIR = P9DIR | 1<<4;           //set P9.4=1 (Transmitter) as output
      P9DIR = P9DIR & ~(1<<5);        //set P9.5=0 (Receiver) as input
      P9SEL = P9SEL | 1<<4 | 1<<5;    //set P9.4=1 & P9.5=1 as TX/RX


      UCA2CTL1 = UCA2CTL1 | 0x01;     //UCSWRST = 1 (bit0 = 1)   //page 417

      UCA2CTL1 = (UCA2CTL1 | 1<<7) & ~(1<<6);       //select SMCLK as BaudRate source clock (bit6 = 1; bit7 = 1)

      //calculate registers for the BaudRate +

      UCA2MCTL = UCA2MCTL | 0x01;       //UCOS16 = 1
      //calculate UCBR
      n = OSC_FREQ / baud;
      UCA2BR0 = (int)(n/16) % 256;
      UCA2BR1 = (int)(n/16) / 256;

      //caluclate UCBRS
      x = (int)((n-(int)(n))*8+0.5);
      UCA2MCTL = UCA2MCTL | (int)(x)<<1;

      //caluclate UCBRF
      x = (int)(((n/16 - (int)(n/16))*16) + 0.5);
      UCA2MCTL = UCA2MCTL | (int)(x)<<4;

      //calculate registers for the BaudRate -

      UCA2CTL0 = 0x00;                  //Set Format Frame: 8 bit data; no parity; 1 stop bit

      UCA2CTL1 = UCA2CTL1 & ~(0x01);    //UCSWRST = 0 (bit0 = 0) //page 417
    }

    unsigned char UART2_Receive()
    {
      if (UCA2IFG & 0x01)
        return UCA2RXBUF;
      else
        return 0;
    }

    void UART2_Transmit (unsigned char data)
    {
      while (!(UCA2IFG & 0x02));
      UCA2TXBUF = data;
    }

    void Terminal_NewLine()
    {
      UART2_Transmit(10);  //down arrow
      UART2_Transmit(13);  //home
    }

    void Terminal_Write(unsigned char *a)
    {
      while (*a)
        UART2_Transmit(*a++);
    }

    void Terminal_WriteLn(unsigned char *a)
    {
      while (*a)
        UART2_Transmit(*a++);
      Terminal_NewLine();
    }

    void Terminal_WriteNum(unsigned long a)
    {
      int br=0, num[10];
      do
      {
        num[br] = a%10;
        br++;
        a = a/10;
      }
      while (a);
      for (br=br-1;br>=0; br--)
        UART2_Transmit(num[br] + 48);
    }

  //UART2 & HyperTerminal functions -

  //UART3 (USB) functions +

    void UART3_Init(unsigned long baud)
    {
      float n, x;
      P10DIR = P10DIR | 1<<4;           //set P10.4=1 (Transmitter) as output
      P10DIR = P10DIR & ~(1<<5);        //set P10.5=0 (Receiver) as input
      P10SEL = P10SEL | 1<<4 | 1<<5;    //set P10.4=1 & P10.5=1 as TX/RX


      UCA3CTL1 = UCA3CTL1 | 0x01;     //UCSWRST = 1 (bit0 = 1)   //page 417

      UCA3CTL1 = (UCA3CTL1 | 1<<7) & ~(1<<6);       //select SMCLK as BaudRate source clock (bit6 = 1; bit7 = 1)

      //calculate registers for the BaudRate +

      UCA3MCTL = UCA3MCTL | 0x01;       //UCOS16 = 1
      //calculate UCBR
      n = OSC_FREQ / baud;
      UCA3BR0 = (int)(n/16) % 256;
      UCA3BR1 = (int)(n/16) / 256;

      //caluclate UCBRS
      x = (int)((n-(int)(n))*8+0.5);
      UCA3MCTL = UCA3MCTL | (int)(x)<<1;

      //caluclate UCBRF
      x = (int)(((n/16 - (int)(n/16))*16) + 0.5);
      UCA3MCTL = UCA3MCTL | (int)(x)<<4;

      //calculate registers for the BaudRate -

      UCA3CTL0 = 0x00;                  //Set Format Frame: 8 bit data; no parity; 1 stop bit

      UCA3CTL1 = UCA3CTL1 & ~(0x01);    //UCSWRST = 0 (bit0 = 0) //page 417
    }

    unsigned char UART3_Receive()
    {
      if (UCA3IFG & 0x01)
        return UCA3RXBUF;
      else
        return 0;
    }

    void UART3_Transmit (unsigned char data)
    {
      while (!(UCA3IFG & 0x02));
      UCA3TXBUF = data;
    }

  //UART3 (USB) functions -

  //SPI2 & LCD functions +

    void SPI2_Init(unsigned long freq)
    {
      unsigned int prescaler;
      UCB2CTL1 = UCB2CTL1 | 0x01;         //UCSWRST = 1 (reset state enable)

      //set format:
      //set clock polarity and phase (UCCKPH = 0; UCCKPL = 1) --> bit7=0; bit6=1
      //MSB first; bit0 (UCSYNC) = 1; bit3 (master mode) = 1; --> bit5=1; bit4=0; bit3 =3; bit0=1;
      //set mode: 3pin SPI --> bit1=0; bit2=3;
      //UCB2CTL0 = 0b01101001 = 0x69
      UCB2CTL0 = 0x69;

      UCB2CTL1 = UCB2CTL1 | 0x80;                     //clock source - SMCLK

      prescaler = OSC_FREQ / freq;
      UCB2BR0 = prescaler % 256;
      UCB2BR1 = prescaler / 256;

      P9SEL = P9SEL | 0x0E;               //P9.0/P9.1/P9.2/P9.3 set as alternate function
      P9DIR = P9DIR | 0x0B;               //P9.0/P9.1/P9.3 set as output

      UCB2CTL1 = UCB2CTL1 & ~(0x01);       //UCSWRST = 0 (reset state disable)
    }

    unsigned char SPI2_Receive()
    {
      if (UCB2IFG & 0x01)
        return UCB2RXBUF;
      else
        return 0;
    }

    void SPI2_Transmit(unsigned char data)
    {
      while (!(UCB2IFG & 0x02));
      UCB2TXBUF = data;
    }

    void LCD_Init()
    {
      P9OUT = P9OUT & 0xBF;         //LCD_RES = 0   //~(1<<6)
      Delay(1000);
      P9OUT = P9OUT | 0x40;         //LCD_RES = 1   //1<<6
      Delay(1000);

      //from Dimitar's PIC source +
        //Send sequence of command +
          LCD_Send(0x21, SEND_CMD);  // LCD Extended Commands.
          //Extended commands +
            LCD_Send(0xC8, SEND_CMD);  // Set Contrast.
            LCD_Send(0x04 | !!(LCD_START_LINE_ADDR&(0x40)), SEND_CMD);      // Set Temp S6 for start line
            LCD_Send(0x40 | (LCD_START_LINE_ADDR & ((0x40)-1)), SEND_CMD);  // Set Temp S[5:0] for start line
            LCD_Send(0x12, SEND_CMD);  // LCD bias mode 1:68.
          //Extended commands -
          LCD_Send(0x20, SEND_CMD);  // LCD Standard Commands, Horizontal addressing mode.
          LCD_Send(0x08, SEND_CMD);  // LCD blank
          LCD_Send(0x0C, SEND_CMD);  // LCD in normal mode.
        //Send sequence of command -
      //from Dimitar's PIC source -

      //Optional commands +
        LCD_Contrast(CONTRAST);   //Increase contrast
        LCD_Clear();              //Clear Screen
        LCD_Set_Position(0, 0);   //0, 0 coordinates of the display
      //Optional Commands -
    }

    void LCD_Send(unsigned char data, unsigned char cd)
    {
      P9OUT = P9OUT & ~(0x01);

      if (cd == SEND_CHR)         //check for sending char/command
        P9OUT = P9OUT | 0x80;     //sending char                    //1<<7
      else
        P9OUT = P9OUT & 0x7F;     //sending command                 //~(1<<7)

      SPI2_Transmit(data);        //transmitting char/command

      P9OUT = P9OUT | 0x01;
    }

    void LCD_Clear_Memory()
    {
      for (index=0; index<504; index++)
        LCDMemory[index] = 0;
      index = 0;
    }

    /*
      mode indicates inverse display:
      (shows black pixels as white and white pixels as black)

      the 1 in binary num indicates which of the lines
      must/must not be inverted
      ex. num = 13 --> 0b001101 --> lines 0, 2, 3 will be inverted
    */
    void LCD_Update(unsigned char mode, unsigned char num)
    {
      char x, y;
      for(y=0; y<48/8; y++)
      {
        LCD_Send(0x80, SEND_CMD);     //x - home
        LCD_Send(0x40|y, SEND_CMD);   //y - next line

        if (((mode==inverse) && !(num & 1<<y)) || ((mode != inverse) && (num & 1<<y)))
          for(x=0; x<84; x++)
            LCD_Send(LCDMemory[y*84+x], SEND_CHR);
        else
          for(x=0; x<84; x++)
            LCD_Send(~LCDMemory[y*84+x], SEND_CHR);
      }
    }

    void LCD_Clear()
    {
      LCD_Clear_Memory();
      LCD_Update(1, 0x00);
    }

    void LCD_Contrast(unsigned char contrast)
    {
        LCD_Send(0x21, SEND_CMD);
        //LCD Extended Commands +
          LCD_Send(0x80 | contrast, SEND_CMD);  // Set LCD Vop (Contrast).
        //LCD Extended Commands -
        LCD_Send(0x20, SEND_CMD);         //LCD Standard Commands, horizontal addressing mode
    }

    void LCD_Set_Position(unsigned char x, unsigned int y)
    {
      //Optional calculations +
        //in case the position is out of the screen:
        //if X > 83 it goes to the next (or more) line.
        //if Y > 5 it return to the 0, 1, 2... line
          y = y + x/84;
          y = y % 6;
          x = x % 84;
      //Optional calculations -
      index = 84*y + x;

      //send cursor to x, y coordinates +
        LCD_Send(0x80 | x, SEND_CMD);
        LCD_Send(0x40 | y, SEND_CMD);
      //send cursor to x, y coordinates -
    }

    void LCD_Write(unsigned char *a)
    {
      unsigned char i;
      while (*a)
      {
        for (i=0; i<5; i++)
          LCDMemory[index++] = FontLookup[*a-32][i];
        LCDMemory[index++] = 0x00;    //space between signs
        a++;
      }
    }

    void LCD_WriteLn(unsigned char *a)
    {
      unsigned char i;
      while (*a)
      {
        for (i=0; i<5; i++)
          LCDMemory[index++] = FontLookup[*a-32][i];
        LCDMemory[index++] = 0x00;    //space between signs
        a++;
      }
      index = (index + 83)/84 * 84;   //move to the next line only if there is at least 1 sign
    }

    void LCD_NewLine()
    {
      index = (index + 84)/84 * 84;   //move to the next line even there isn't sign on the line
    }

    void LCD_WriteNum(unsigned long a)
    {
      int br=0, num[10], i;
      do
      {
        num[br] = a%10;
        br++;
        a = a/10;
      }
      while (a);
      for (br=br-1;br>=0; br--)
      {
        for (i=0; i<5; i++)
          LCDMemory[index++] = FontLookup[num[br] + 16][i];
        LCDMemory[index++] = 0x00;    //space between signs
      }
    }

    void LCD_Char_XY(char x,char y, unsigned char ch, char mode)
    {
      unsigned char i, j;
      j = 84*y+6*x;
      if ((x<=13) && (y<=5))
        for (i=0; i<5; i++)
          LCDMemory[j++] = FontLookup[ch-32][i];
      LCDMemory[j++] = 0x00;    //space between signs

      //if mode=1 --> place the cursor at position after the placed sign
      if (mode)
        index = j;
    }

    void LCD_Menu(char p)
    {
      LCD_Set_Position(0, 0);
      LCD_Clear_Memory();

      switch (p)
      {
        case 1:
          LCD_WriteLn("   MENU 1/2");
          LCD_WriteLn("1) LED 1");
          LCD_WriteLn("2) LED 2");
          LCD_WriteLn("3) Microphone");
          LCD_WriteLn("4) Extension");
          LCD_WriteLn("5) UExt");
          LCD_Update(1, 1<<line);
        break;
        case 2:
          LCD_WriteLn("   MENU 2/2");
          LCD_WriteLn("Inverse LCD");
          LCD_WriteLn("Test MMC");
		LCD_WriteLn("Test RS & USB");
          LCD_Update(1, 1<<line);
        break;
        case 11:
          LCD_WriteLn("  TEST LED 1");
          LCD_WriteLn("Turn ON");
          LCD_WriteLn("Turn OFF");
          LCD_WriteLn("Blink");
          LCD_WriteLn("Exit Test");
          LCD_Update(1, 1<<line);
        break;
        case 12:
          LCD_WriteLn("  TEST LED 2");
          LCD_WriteLn("Turn ON");
          LCD_WriteLn("Turn OFF");
          LCD_WriteLn("Blink");
          LCD_WriteLn("Exit Test");
          LCD_Update(1, 1<<line);
        break;
        case 13:
          LCD_WriteLn("   TEST ADC");
          LCD_WriteLn("Exit Test");
          LCD_Update(1, 1<<line);
        break;
	   case 22:
	  	LCD_WriteLn("   TEST MMC");
	   break;
	   case 23:
		LCD_WriteLn("  Echo RS&USB");
		LCD_WriteLn("Exit Test");
		LCD_Update(1, 1<<1);
	   break;
      }
    }

    void LCD_Inverse()
    {
	inverse = (inverse+1)%2;
    }
  //SPI2 & LCD functions -

  //ADC + "DAC" (TimerA) functions +

    void ADC12_Init()
    {
      P8SEL = P8SEL | 0x02;                   //P8.1 --> ADC function
      P8DIR = P8DIR | 0x02;                   //PORT8.1 = 1 (ADC_OUT) set as output
      P6SEL = P6SEL | 0x80;                   //P6.7 --> ADC function
      P6DIR = P6DIR & ~0x80;                  //PORT6.7 = 0 (ADC_IN)  set as input

      ADC12CTL0 = ADC12CTL0 & ~(0x02);        //ADC12ENC = 0


      ADC12CTL0 = ADC12CTL0 | 1<<4;           //ADC12ON = 1
      ADC12CTL0 = ADC12CTL0 | 0x9900;         //ADC12SHT(0/1) = 0b1001 (Cycles = 384)

      ADC12CTL1 = ADC12CTL1 | 0x08 | 1<<4;    //ADC12SEL bits = 0b11 (Source clock - SMCLK)
      ADC12CTL1 = ADC12CTL1 & ~(0x02 | 0x04); //ADC12CONSEQ = 0b00 --> Single Chanel, Single Conversion
      ADC12CTL1 = ADC12CTL1 | 0x7000;         //select first (in order) register (7)
      ADC12CTL1 = ADC12CTL1 | 1<<9;           //SAMPCON - sampling timer

      ADC12CTL2 = ADC12CTL2 & ~(0x08);        //ADC12DF=0   --> unsigned format

      ADC12CTL2 = ADC12CTL2 & ~(1<<4 | 1<<5); //ADC12RES = 0b00 (8 bit mode resolution)

      ADC12MCTL7 = 0xF7;                      //End of Sequence --> Bit7 = 1; ADC12SREF = 0b111; ADC12INCH = 0b0111;


      ADC12CTL0 = ADC12CTL0 | 0x02;           //ADC12ENC = 1
    }

    unsigned int ADC12_Receive()
    {
      ADC12CTL0 = ADC12CTL0 | 0x01;           //start conversion ON
      while (ADC12CTL1 & 0x01);               //wait while BUSY
      return ADC12MEM7;
    }

    void TIMER_A_Init()
    {
      TA0CTL = (TA0CTL | 1<<9) & ~(1<<8);     //TimerA clock source is SMCLK (bit8=0, bit9=1)

      TA0CTL = (TA0CTL | 1<<4) & ~(1<<5);     //Mode Control set to UP mode (MC bits = 0b01)
      TA0CCTL0 = TA0CCTL0 | 0x00A0;           //OUTMODE0 = 0b001 (set mode)
      TA0CCTL1 = TA0CCTL1 | 0x00C0;           //OUTMODE1 = 0b110 (set/reset mode)

      TA0CCR0 = 0x00;                         //turn off timer
    }

  //ADC + "DAC" (TimerA) functions -

  //SPI0 & MMC Test +

    void SPI0_Init(unsigned long freq)
    {
      unsigned int prescaler;
      UCB0CTL1 = UCB0CTL1 | 0x01;         //UCSWRST = 1 (reset state enable)

      //set format:
      //set clock polarity and phase (UCCKPH = 0; UCCKPL = 1) --> bit7=0; bit6=1
      //MSB first; bit0 (UCSYNC) = 1; bit3 (master mode) = 1; --> bit5=1; bit4=0; bit3 =3; bit0=1;
      //set mode: 3pin SPI --> bit1=0; bit2=3;
      //UCB0CTL0 = 0b01101001 = 0x69
      UCB0CTL0 = 0x69;

      UCB0CTL1 = UCB0CTL1 | 0x80;                     //clock source - SMCLK

      prescaler = OSC_FREQ / freq;
      UCB0BR0 = prescaler % 256;
      UCB0BR1 = prescaler / 256;

      P3SEL |= 0x0E;				//SIMO, SOMI, CLOCK alternative
      P3OUT |= 0x01;				//Chip Select to High
      P3DIR |= 0x0B;				//Chip Select (MMC), SIMO, CLOCK output

      UCB0CTL1 = UCB0CTL1 & ~(0x01);       //UCSWRST = 0 (reset state disable)
    }

    unsigned char SPI0_Receive()
    {
      if (UCB0IFG & 0x01)
        return UCB0RXBUF;
      else
        return 0;
    }

    void SPI0_Transmit(unsigned char data)
    {
      while (!(UCB0IFG & 0x02));
      UCB0TXBUF = data;
    }


	/****************************************************************************************/
	/*  Function name: Test_MMC					                       			   */
	/*  	Parameters                                                                		   */
	/*          Input   :  No				                                		   */
	/*          Output  :  No	                                                      	   */
	/*     Action: Check SD/MMC card connection, write, read and verify sector of  512 bytes*/
	/****************************************************************************************/
	void Test_MMC(void){
		LCD_Menu(22);
		Terminal_Write("\n\r *** SD/MMC Test: *** ");
		
		// 3. Card initialization:
		if((initMMC() == MMC_INIT_SUCCESS)){	// Card initialization completed successfully
			Terminal_Write("\n\r Card initialization completed successfully! ");
		}
		else{	// Card initialization Fail
			Terminal_Write("\n\r Card initialization Fail! ");
			Terminal_Write("\n\r Please repeat test again! \n\r");
			LCD_WriteLn("MMC:FAIL");
			MMC_Flag = 0;
			return;
		}
		// 4. Read OCR register!!!
		if((mmcReadRegister(58,4)== MMC_ReadREG_SUCCESS)){ // OCR Register read completed successfully
			Terminal_Write("\n\r OCR Register Read  operation completed successfully!");
			unsigned long int OCR_value = 0x00000000;
			
			for(unsigned char i=0; i<=3; i++){
				OCR_value = OCR_value | mmcReg_buffer[15-i];
				if(i==3)	break;
				OCR_value = OCR_value << 8;
			}
			if(mmcReg_buffer[15]&0x80){	// Check if status bit is set? [p.g. 24]
				Terminal_Write("\n\r -> Card Power-Up procedure has been finished!");
				// By experiments I found that card operation voltage diapason is (2.7 - 3.6)V!
				// Below this voltage, card can not be initialized! So other two values of OCR register can't be read!
				// If card supply voltage is below 1.5V, CARD WILL BE BURST!!! I BURST ONE DURING THE EXPERIMENTS!!!
				if(OCR_value == 0x80FF8000)	Terminal_Write("\n\r -> SD/MMC Power Supply: (2.7 - 3.6)V");
				else if(OCR_value == 0x80007F00) Terminal_Write("\n\r -> SD/MMC Power Supply: (2.0 - 2.6)V");
				else if(OCR_value == 0x80000080) Terminal_Write("\n\r -> SD/MMC Power Supply: (1.65 - 1.95)V");
				else Terminal_Write("\n\r -> Error in OCR_value reading!!!");
			}
			else{
				Terminal_Write("\n\r -> Card Power-Up procedure hasn't been finished yet!");
				LCD_WriteLn("MMC:FAIL");
				MMC_Flag = 0;
				return;
			}
		}
		else{	// Error in OCR register reading
			Terminal_Write("\n\r OCR Register Read operation FAIL!\n\r");
			LCD_WriteLn("MMC:FAIL");
			MMC_Flag = 0;
			return;
		}
		// 5. Read CID Register
		if((mmcReadRegister(10,16)== MMC_ReadREG_SUCCESS)){ // CID Register read completed successfully
			Terminal_Write("\n\r CID Register Read  operation completed successfully!");
		}
		else{	// Error in CID register reading
			Terminal_Write("\n\r CID Register Read operation FAIL!\n\r");
			LCD_WriteLn("MMC:FAIL");
			MMC_Flag = 0;
			return;
		}
		//6. Read CSD Register and calculate card size
		if((mmcReadRegister(9,16)== MMC_ReadREG_SUCCESS)){ // CSD Register read completed successfully
			Terminal_Write("\n\r CSD Register Read  operation completed successfully!");
			char mmc_size;
	
			mmc_size = mmcCapacity();
			if (mmc_size == mmcMemory_less_than_64MB)		Terminal_Write("\n\r -> MMC Memory: less than 64MB!");
			else if(mmc_size == mmcMemory_64MB)				Terminal_Write("\n\r -> MMC Memory: 64MB!");
			else if(mmc_size == mmcMemory_128MB)			Terminal_Write("\n\r -> MMC Memory: 128MB!");
			else if(mmc_size == mmcMemory_256MB)			Terminal_Write("\n\r -> MMC Memory: 256MB!");
			else if(mmc_size == mmcMemory_512MB)			Terminal_Write("\n\r -> MMC Memory: 512MB!");
			else if(mmc_size == mmcMemory_1GB)				Terminal_Write("\n\r -> MMC Memory: 1GB!");
			else if(mmc_size == mmcMemory_2GB)				Terminal_Write("\n\r -> MMC Memory: 2GB!");
			else if(mmc_size == mmcMemory_4GB)				Terminal_Write("\n\r -> MMC Memory: 4GB!");
			else if(mmc_size == mmcMemory_more_than_4GB)	Terminal_Write("\n\r -> MMC Memory: more than 4GB!");
			else{
				Terminal_Write("\n\r MMC Memory: Calculation ERROR!");
				LCD_Write("MMC:FAIL");
				MMC_Flag = 0;
				return;
			}			
		}
		else{	// Error in CSD register reading
			Terminal_Write("\n\r CSD Register Read operation FAIL!\n\r");
			LCD_WriteLn("MMC:FAIL");
			MMC_Flag = 0;
			return;
		}
	
		// 7. Write Sector of 512 bytes
		for(int index = 0; index<512; index++){
			if(index<128)		mmc_write_buffer[index] = 0x33;
			else if(index<256)	mmc_write_buffer[index] = 0x55;
			else if(index<384)	mmc_write_buffer[index] = 0xAA;
			else				mmc_write_buffer[index] = 0xCC;
		}	
		// Full block number 10(its start address is 131072) with mmc_write_buffer
		if(mmcWriteBlock(131072)== MMC_WRITE_SUCCESS){
			Terminal_Write("\n\r MMC Write Block completed successfully!");
		}
		else{
			Terminal_Write("\n\r MMC Write Block FAIL!\n\r");
			LCD_WriteLn("MMC:FAIL");
			MMC_Flag = 0;
			return;
		}
		// 8. Read the same Sector of 512 bytes and verify it!
		if(mmcRead_VerifyBlock(131072,512)==MMC_Read_Write_Verification_Success){
			Terminal_Write("\n\r MMC Read Block completed successfully!");
			Terminal_Write("\n\r MMC Read/Write Verification completed successfully!");
		}
		else{
			Terminal_Write("\n\r MMC Read Block FAIL!\n\r");
			Terminal_Write("\n\r MMC Read/Write Verification FAIL!\n\r");
			LCD_WriteLn("MMC:FAIL");
			MMC_Flag = 0;
			return;
		}
	
		Terminal_Write("\n\r SD/MMC Test completed SUCCESSFULLY!\n\r");
		LCD_WriteLn("MMC: Ok!");
		MMC_Flag = 1;
	}	// SD/MMC test End


  //SPI0 & MMC Test -

  //Extension functions +

    //Extension pins set as i/o functions and as input
    void Extension_Init()
    {
      P1SEL = P1SEL & ~(P1MASK);
      P1DIR = P1DIR & ~(P1MASK);

      P2SEL = P2SEL & ~(P2MASK);
      P2DIR = P2DIR & ~(P2MASK);

      P3SEL = P3SEL & ~(P3MASK);
      P3DIR = P3DIR & ~(P3MASK);

      P4SEL = P4SEL & ~(P4MASK);
      P4DIR = P4DIR & ~(P4MASK);

      P5SEL = P5SEL & ~(P5MASK);
      P5DIR = P5DIR & ~(P5MASK);

      P6SEL = P6SEL & ~(P6MASK);
      P6DIR = P6DIR & ~(P6MASK);

      P7SEL = P7SEL & ~(P7MASK);
      P7DIR = P7DIR & ~(P7MASK);

      P8SEL = P8SEL & ~(P8MASK);
      P8DIR = P8DIR & ~(P8MASK);

      P9SEL = P9SEL & ~(P9MASK);
      P9DIR = P9DIR & ~(P9MASK);

      P10SEL = P10SEL & ~(P10MASK);
      P10DIR = P10DIR & ~(P10MASK);

      P11SEL = P11SEL & ~(P11MASK);
      P11DIR = P11DIR & ~(P11MASK);

    }

    void Extension_Locate(unsigned char p, unsigned char port)
    {
      unsigned char q;
      for (q=0; q<=7; q++)
      {
        if (p%2)
        {
          Terminal_NewLine();

          //ex. "PORT3.2"
          Terminal_Write("PORT");
          Terminal_WriteNum(port);
          UART2_Transmit('.');
          Terminal_WriteNum(q);

          //ex. " (35)"
          Terminal_Write(" (");
          Terminal_WriteNum(a[port][q]);
          Terminal_Write(")");
        }
        p = p/2;
      }
    }

    void Extension_Locate_PIN(unsigned char port1, unsigned char pin1, unsigned port2, unsigned char pin2)
    {
      unsigned char q;
      for (q=0; q<=7; q++)
      {
        if (pin2%2)
        {
          Terminal_NewLine();

          //ex. "PORT3.0 to PORT3.2"
          Terminal_Write("PORT"); Terminal_WriteNum(port1); UART2_Transmit('.'); Terminal_WriteNum(pin1);
          Terminal_Write(" to ");
          Terminal_Write("PORT"); Terminal_WriteNum(port2); UART2_Transmit('.'); Terminal_WriteNum(q);

          //ex. " (33 to 35)"
          Terminal_Write(" (");
          Terminal_WriteNum(a[port1][pin1]);
          Terminal_Write(" to ");
          Terminal_WriteNum(a[port2][q]);
          Terminal_Write(")");
        }
        pin2 = pin2 / 2;
      }
    }

    void Extension_Locate_PullPin(int x, int y)
    {
      Terminal_NewLine();
      Terminal_Write("PORT");
      Terminal_WriteNum(x);
      UART2_Transmit('.');
      Terminal_WriteNum(y);
      Terminal_Write(" (");
      Terminal_WriteNum(a[x][y]);
      UART2_Transmit(')');
      f=0;
    }

    unsigned char Extension_Check()
    {
      int i;

      //Test PIN to GND +

      Terminal_Write("Test GND");
      f=1;
      P1DIR = P1DIR | 0x80;         //P1.7 (PullPin) set as output
      P1OUT = P1OUT | 0x80;         //P1.7 = 1
      Delay(HOLD_TIME);
      if (P1MASK & ~P1IN) {Extension_Locate(P1MASK & ~P1IN, 1); f=0;}
      if (P2MASK & ~P2IN) {Extension_Locate(P2MASK & ~P2IN, 2); f=0;}
      if (P3MASK & ~P3IN) {Extension_Locate(P3MASK & ~P3IN, 3); f=0;}
      if (P4MASK & ~P4IN) {Extension_Locate(P4MASK & ~P4IN, 4); f=0;}
      if (P5MASK & ~P5IN) {Extension_Locate(P5MASK & ~P5IN, 5); f=0;}
      if (P6MASK & ~P6IN) {Extension_Locate(P6MASK & ~P6IN, 6); f=0;}
      if (P7MASK & ~P7IN) {Extension_Locate(P7MASK & ~P7IN, 7); f=0;}
      if (P8MASK & ~P8IN) {Extension_Locate(P8MASK & ~P8IN, 8); f=0;}
      if (P9MASK & ~P9IN) {Extension_Locate(P9MASK & ~P9IN, 9); f=0;}
      if (P10MASK & ~P10IN) {Extension_Locate(P10MASK & ~P10IN, 10); f=0;}
      if (P11MASK & ~P11IN) {Extension_Locate(P11MASK & ~P11IN, 11); f=0;}
      g_f = g_f && f;
      if (f)
        Terminal_WriteLn(" - OK");
      else
        Terminal_NewLine();

      //Test PIN to GND -

      Terminal_NewLine();

      //Test PIN to VCC +

      Terminal_Write("Test VCC");
      f=1;
      P1DIR = P1DIR | 0x80;         //P1.7 (PullPin) set as output
      P1OUT = P1OUT & 0x7F;         //P1.7 = 0
      Delay(HOLD_TIME);
      if (P1MASK & P1IN) {Extension_Locate(P1MASK & P1IN, 1); f=0;}
      if (P2MASK & P2IN) {Extension_Locate(P2MASK & P2IN, 2); f=0;}
      if (P3MASK & P3IN) {Extension_Locate(P3MASK & P3IN, 3); f=0;}
      if (P4MASK & P4IN) {Extension_Locate(P4MASK & P4IN, 4); f=0;}
      if (P5MASK & P5IN) {Extension_Locate(P5MASK & P5IN, 5); f=0;}
      if (P6MASK & P6IN) {Extension_Locate(P6MASK & P6IN, 6); f=0;}
      if (P7MASK & P7IN) {Extension_Locate(P7MASK & P7IN, 7); f=0;}
      if (P8MASK & P8IN) {Extension_Locate(P8MASK & P8IN, 8); f=0;}
      if (P9MASK & P9IN) {Extension_Locate(P9MASK & P9IN, 9); f=0;}
      if (P10MASK & P10IN) {Extension_Locate(P10MASK & P10IN, 10); f=0;}
      if (P11MASK & P11IN) {Extension_Locate(P11MASK & P11IN, 11); f=0;}
      g_f = g_f && f;
      if (f)
        Terminal_WriteLn(" - OK");
      else
        Terminal_NewLine();

      //Test PIN to VCC -

      Terminal_NewLine();

      //Test PIN to PullPin +

        Terminal_Write("Test PullPin (P1.7 - 24)");
        f=1;
        //Running 0 +
        P1DIR = P1DIR | 0x80;
        P1OUT = P1OUT | 0x80;

        for (i=0; i<=7; i++)
        {
          test_const = 1<<i;
          if ((i!=7) && (P1MASK & test_const))
          {
            P1OUT = P1OUT & ~(test_const);
            P1DIR = P1DIR | test_const;
            for (counter=TIMEOUT; counter; counter--)        //counter used as timeout
            {
              Delay(PULL_HOLD_TIME);
	      if (P1IN & test_const)
//              if (!(P1IN & 0x80))
              {
                Extension_Locate_PullPin(1, i);
                break;
              }
            }
            P1DIR = P1DIR & ~(test_const);
          }
        }

        for (i=0; i<=7; i++)
        {
          test_const = 1<<i;
          if (P2MASK & test_const)
          {
            P2OUT = P2OUT & ~(test_const);
            P2DIR = P2DIR | test_const;
            for (counter=TIMEOUT; counter; counter--)        //counter used as timeout
            {
              Delay(PULL_HOLD_TIME);
	      if (P2IN & test_const)
//              if (!(P1IN & 0x80))
              {
                Extension_Locate_PullPin(2, i);
                break;
              }
            }
            P2DIR = P2DIR & ~(test_const);
          }
        }

        for (i=0; i<=7; i++)
        {
          test_const = 1<<i;
          if (P3MASK & test_const)
          {
            P3OUT = P3OUT & ~(test_const);
            P3DIR = P3DIR | test_const;
            for (counter=TIMEOUT; counter; counter--)        //counter used as timeout
            {
              Delay(PULL_HOLD_TIME);
	      if (P3IN & test_const)
//              if (!(P1IN & 0x80))
              {
                Extension_Locate_PullPin(3, i);
                break;
              }
            }
            P3DIR = P3DIR & ~(test_const);
          }
        }

        for (i=0; i<=7; i++)
        {
          test_const = 1<<i;
          if (P4MASK & test_const)
          {
            P4DIR = P4DIR | test_const;
            P4OUT = P4OUT & ~(test_const);
            for (counter=TIMEOUT; counter; counter--)        //counter used as timeout
            {
              Delay(PULL_HOLD_TIME);
	      if (P4IN & test_const)
//              if (!(P1IN & 0x80))
              {
                Extension_Locate_PullPin(4, i);
                break;
              }
            }
            P4DIR = P4DIR & ~(test_const);
          }
        }
	
        for (i=0; i<=7; i++)
        {
          test_const = 1<<i;
          if (P5MASK & test_const)
          {
            P5OUT = P5OUT & ~(test_const);
            P5DIR = P5DIR | test_const;
            for (counter=TIMEOUT; counter; counter--)        //counter used as timeout
            {
              Delay(PULL_HOLD_TIME);
	      if (P5IN & test_const)
//              if (!(P1IN & 0x80))
              {
                Extension_Locate_PullPin(5, i);
                break;
              }
            }
            P5DIR = P5DIR & ~(test_const);
          }
        }

        for (i=0; i<=7; i++)
        {
          test_const = 1<<i;
          if (P6MASK & test_const)
          {
            P6OUT = P6OUT & ~(test_const);
            P6DIR = P6DIR | test_const;
            for (counter=TIMEOUT; counter; counter--)        //counter used as timeout
            {
              Delay(PULL_HOLD_TIME);
	      if (P6IN & test_const)
//              if (!(P1IN & 0x80))
              {
                Extension_Locate_PullPin(6, i);
                break;
              }
            }
            P6DIR = P6DIR & ~(test_const);
          }
        }

        for (i=0; i<=7; i++)
        {
          test_const = 1<<i;
          if (P7MASK & test_const)
          {
            P7OUT = P7OUT & ~(test_const);
            P7DIR = P7DIR | test_const;
            for (counter=TIMEOUT; counter; counter--)        //counter used as timeout
            {
              Delay(PULL_HOLD_TIME);
	      if (P7IN & test_const)
//              if (!(P1IN & 0x80))
              {
                Extension_Locate_PullPin(7, i);
                break;
              }
            }
            P7DIR = P7DIR & ~(test_const);
          }
        }

        for (i=0; i<=7; i++)
        {
          test_const = 1<<i;
          if (P8MASK & test_const)
          {
            P8OUT = P8OUT & ~(test_const);
            P8DIR = P8DIR | test_const;
            for (counter=TIMEOUT; counter; counter--)        //counter used as timeout
            {
              Delay(PULL_HOLD_TIME);
	      if (P8IN & test_const)
//              if (!(P1IN & 0x80))
              {
                Extension_Locate_PullPin(8, i);
                break;
              }
            }
            P8DIR = P8DIR & ~(test_const);
          }
        }

        for (i=0; i<=7; i++)
        {
          test_const = 1<<i;
          if (P9MASK & test_const)
          {
            P9OUT = P9OUT & ~(test_const);
            P9DIR = P9DIR | test_const;
            for (counter=TIMEOUT; counter; counter--)        //counter used as timeout
            {
              Delay(PULL_HOLD_TIME);
	      if (P9IN & test_const)
//              if (!(P1IN & 0x80))
              {
                Extension_Locate_PullPin(9, i);
                break;
              }
            }
            P9DIR = P9DIR & ~(test_const);
          }
        }

        for (i=0; i<=7; i++)
        {
          test_const = 1<<i;
          if (P10MASK & test_const)
          {
            P10OUT = P10OUT & ~(test_const);
            P10DIR = P10DIR | test_const;
            for (counter=TIMEOUT; counter; counter--)        //counter used as timeout
            {
              Delay(PULL_HOLD_TIME);
	      if (P10IN & test_const)
//              if (!(P1IN & 0x80))
              {
                Extension_Locate_PullPin(10, i);
                break;
              }
            }
            P10DIR = P10DIR & ~(test_const);
          }
        }

        for (i=0; i<=7; i++)
        {
          test_const = 1<<i;
          if (P11MASK & test_const)
          {
            P11OUT = P11OUT & ~(test_const);
            P11DIR = P11DIR | test_const;
            for (counter=TIMEOUT; counter; counter--)        //counter used as timeout
            {
              Delay(PULL_HOLD_TIME);
	      if (P11IN & test_const)
//              if (!(P1IN & 0x80))
              {
                Extension_Locate_PullPin(11, i);
                break;
              }
            }
            P11DIR = P11DIR & ~(test_const);
          }
        }

        g_f = g_f && f;
        if (f)
          Terminal_WriteLn(" - OK");
        else
          Terminal_NewLine();
        //Running 0 -

      //Test PIN to PullPin -

      if (!g_f) return 0;

      Terminal_NewLine();

      //Test PIN to PIN +

      P1OUT = P1OUT & ~(0x80);        //P1.7 (PullPin) = 0
      Terminal_WriteLn("Test PIN to PIN:");

        //Test PORT1 +
          f=1;
          Terminal_Write("Test PORT1");
          for (i=0; i<=7; i++)
            if ((i!=7) && (P1MASK & 1<<i))
            {
              P1DIR = P1DIR | 1<<i;
              P1OUT = P1OUT | 1<<i;
              Delay(HOLD_TIME);
              if (P1MASK & P1IN & ~(1<<i)) {Extension_Locate_PIN(1, i, 1, (P1MASK & P1IN) & ~(1<<i)); f=0;}
              if (P2MASK & P2IN) {Extension_Locate_PIN(1, i, 2, (P2MASK & P2IN)); f=0;}
              if (P3MASK & P3IN) {Extension_Locate_PIN(1, i, 3, (P3MASK & P3IN)); f=0;}
              if (P4MASK & P4IN) {Extension_Locate_PIN(1, i, 4, (P4MASK & P4IN)); f=0;}
              if (P5MASK & P5IN) {Extension_Locate_PIN(1, i, 5, (P5MASK & P5IN)); f=0;}
              if (P6MASK & P6IN) {Extension_Locate_PIN(1, i, 6, (P6MASK & P6IN)); f=0;}
              if (P7MASK & P7IN) {Extension_Locate_PIN(1, i, 7, (P7MASK & P7IN)); f=0;}
              if (P8MASK & P8IN) {Extension_Locate_PIN(1, i, 8, (P8MASK & P8IN)); f=0;}
              if (P9MASK & P9IN) {Extension_Locate_PIN(1, i, 9, (P9MASK & P9IN)); f=0;}
              if (P10MASK & P10IN) {Extension_Locate_PIN(1, i, 10, (P10MASK & P10IN)); f=0;}
              if (P11MASK & P11IN) {Extension_Locate_PIN(1, i, 11, (P11MASK & P11IN)); f=0;}
              P1DIR = P1DIR & ~(1<<i);
              P1OUT = P1OUT & ~(1<<i);
            }
          g_f = g_f && f;
          if (f)
            Terminal_Write(" - OK");
          /*else
            return 0;*/
        //Test PORT1 -

            Terminal_NewLine();

        //Test PORT2 +
          f=1;
          Terminal_Write("Test PORT2");
          for (i=0; i<=7; i++)
            if (P2MASK & 1<<i)
            {
              P2DIR = P2DIR | 1<<i;
              P2OUT = P2OUT | 1<<i;
              Delay(HOLD_TIME);
              if (P1MASK & P1IN) {Extension_Locate_PIN(2, i, 1, (P1MASK & P1IN)); f=0;}
              if (P2MASK & P2IN & ~(1<<i)) {Extension_Locate_PIN(2, i, 2, (P2MASK & P2IN) & ~(1<<i)); f=0;}
              if (P3MASK & P3IN) {Extension_Locate_PIN(2, i, 3, (P3MASK & P3IN)); f=0;}
              if (P4MASK & P4IN) {Extension_Locate_PIN(2, i, 4, (P4MASK & P4IN)); f=0;}
              if (P5MASK & P5IN) {Extension_Locate_PIN(2, i, 5, (P5MASK & P5IN)); f=0;}
              if (P6MASK & P6IN) {Extension_Locate_PIN(2, i, 6, (P6MASK & P6IN)); f=0;}
              if (P7MASK & P7IN) {Extension_Locate_PIN(2, i, 7, (P7MASK & P7IN)); f=0;}
              if (P8MASK & P8IN) {Extension_Locate_PIN(2, i, 8, (P8MASK & P8IN)); f=0;}
              if (P9MASK & P9IN) {Extension_Locate_PIN(2, i, 9, (P9MASK & P9IN)); f=0;}
              if (P10MASK & P10IN) {Extension_Locate_PIN(2, i, 10, (P10MASK & P10IN)); f=0;}
              if (P11MASK & P11IN) {Extension_Locate_PIN(2, i, 11, (P11MASK & P11IN)); f=0;}
              P2DIR = P2DIR & ~(1<<i);
              P2OUT = P2OUT & ~(1<<i);
            }
          g_f = g_f && f;
          if (f)
            Terminal_Write(" - OK");
          /*else
            return 0;*/
        //Test PORT2 -

            Terminal_NewLine();

        //Test PORT3 +
          f=1;
          Terminal_Write("Test PORT3");
          for (i=0; i<=7; i++)
            if (P3MASK & 1<<i)
            {
              P3DIR = P3DIR | 1<<i;
              P3OUT = P3OUT | 1<<i;
              Delay(HOLD_TIME);
              if (P1MASK & P1IN) {Extension_Locate_PIN(3, i, 1, (P1MASK & P1IN)); f=0;}
              if (P2MASK & P2IN) {Extension_Locate_PIN(3, i, 2, (P2MASK & P2IN)); f=0;}
              if (P3MASK & P3IN & ~(1<<i)) {Extension_Locate_PIN(3, i, 3, (P3MASK & P3IN) & ~(1<<i)); f=0;}
              if (P4MASK & P4IN) {Extension_Locate_PIN(3, i, 4, (P4MASK & P4IN)); f=0;}
              if (P5MASK & P5IN) {Extension_Locate_PIN(3, i, 5, (P5MASK & P5IN)); f=0;}
              if (P6MASK & P6IN) {Extension_Locate_PIN(3, i, 6, (P6MASK & P6IN)); f=0;}
              if (P7MASK & P7IN) {Extension_Locate_PIN(3, i, 7, (P7MASK & P7IN)); f=0;}
              if (P8MASK & P8IN) {Extension_Locate_PIN(3, i, 8, (P8MASK & P8IN)); f=0;}
              if (P9MASK & P9IN) {Extension_Locate_PIN(3, i, 9, (P9MASK & P9IN)); f=0;}
              if (P10MASK & P10IN) {Extension_Locate_PIN(3, i, 10, (P10MASK & P10IN)); f=0;}
              if (P11MASK & P11IN) {Extension_Locate_PIN(3, i, 11, (P11MASK & P11IN)); f=0;}
              P3DIR = P3DIR & ~(1<<i);
              P3OUT = P3OUT & ~(1<<i);
            }
          g_f = g_f && f;
          if (f)
            Terminal_Write(" - OK");
          /*else
            return 0;*/
        //Test PORT3 -

            Terminal_NewLine();

        //Test PORT4 +
          f=1;
          Terminal_Write("Test PORT4");
          for (i=0; i<=7; i++)
            if (P4MASK & 1<<i)
            {
              P4DIR = P4DIR | 1<<i;
              P4OUT = P4OUT | 1<<i;
              Delay(HOLD_TIME);
              if (P1MASK & P1IN) {Extension_Locate_PIN(4, i, 1, (P1MASK & P1IN)); f=0;}
              if (P2MASK & P2IN) {Extension_Locate_PIN(4, i, 2, (P2MASK & P2IN)); f=0;}
              if (P3MASK & P3IN) {Extension_Locate_PIN(4, i, 3, (P3MASK & P3IN)); f=0;}
              if (P4MASK & P4IN & ~(1<<i)) {Extension_Locate_PIN(4, i, 4, (P4MASK & P4IN) & ~(1<<i)); f=0;}
              if (P5MASK & P5IN) {Extension_Locate_PIN(4, i, 5, (P5MASK & P5IN)); f=0;}
              if (P6MASK & P6IN) {Extension_Locate_PIN(4, i, 6, (P6MASK & P6IN)); f=0;}
              if (P7MASK & P7IN) {Extension_Locate_PIN(4, i, 7, (P7MASK & P7IN)); f=0;}
              if (P8MASK & P8IN) {Extension_Locate_PIN(4, i, 8, (P8MASK & P8IN)); f=0;}
              if (P9MASK & P9IN) {Extension_Locate_PIN(4, i, 9, (P9MASK & P9IN)); f=0;}
              if (P10MASK & P10IN) {Extension_Locate_PIN(4, i, 10, (P10MASK & P10IN)); f=0;}
              if (P11MASK & P11IN) {Extension_Locate_PIN(4, i, 11, (P11MASK & P11IN)); f=0;}
              P4DIR = P4DIR & ~(1<<i);
              P4OUT = P4OUT & ~(1<<i);
            }
          g_f = g_f && f;
          if (f)
            Terminal_Write(" - OK");
          /*else
            return 0;*/
        //Test PORT4 -

            Terminal_NewLine();

        //Test PORT5 +
          f=1;
          Terminal_Write("Test PORT5");
          for (i=0; i<=7; i++)
            if (P5MASK & 1<<i)
            {
              P5DIR = P5DIR | 1<<i;
              P5OUT = P5OUT | 1<<i;
              Delay(HOLD_TIME);
              if (P1MASK & P1IN) {Extension_Locate_PIN(5, i, 1, (P1MASK & P1IN)); f=0;}
              if (P2MASK & P2IN) {Extension_Locate_PIN(5, i, 2, (P2MASK & P2IN)); f=0;}
              if (P3MASK & P3IN) {Extension_Locate_PIN(5, i, 3, (P3MASK & P3IN)); f=0;}
              if (P4MASK & P4IN) {Extension_Locate_PIN(5, i, 4, (P4MASK & P4IN)); f=0;}
              if (P5MASK & P5IN & ~(1<<i)) {Extension_Locate_PIN(5, i, 5, (P5MASK & P5IN) & ~(1<<i)); f=0;}
              if (P6MASK & P6IN) {Extension_Locate_PIN(5, i, 6, (P6MASK & P6IN)); f=0;}
              if (P7MASK & P7IN) {Extension_Locate_PIN(5, i, 7, (P7MASK & P7IN)); f=0;}
              if (P8MASK & P8IN) {Extension_Locate_PIN(5, i, 8, (P8MASK & P8IN)); f=0;}
              if (P9MASK & P9IN) {Extension_Locate_PIN(5, i, 9, (P9MASK & P9IN)); f=0;}
              if (P10MASK & P10IN) {Extension_Locate_PIN(5, i, 10, (P10MASK & P10IN)); f=0;}
              if (P11MASK & P11IN) {Extension_Locate_PIN(5, i, 11, (P11MASK & P11IN)); f=0;}
              P5DIR = P5DIR & ~(1<<i);
              P5OUT = P5OUT & ~(1<<i);
            }
          g_f = g_f && f;
          if (f)
            Terminal_Write(" - OK");
          /*else
            return 0;*/
        //Test PORT5 -

            Terminal_NewLine();

        //Test PORT6 +
          f=1;
          Terminal_Write("Test PORT6");
          for (i=0; i<=7; i++)
            if (P6MASK & 1<<i)
            {
              P6DIR = P6DIR | 1<<i;
              P6OUT = P6OUT | 1<<i;
              Delay(HOLD_TIME);
              if (P1MASK & P1IN) {Extension_Locate_PIN(6, i, 1, (P1MASK & P1IN)); f=0;}
              if (P2MASK & P2IN) {Extension_Locate_PIN(6, i, 2, (P2MASK & P2IN)); f=0;}
              if (P3MASK & P3IN) {Extension_Locate_PIN(6, i, 3, (P3MASK & P3IN)); f=0;}
              if (P4MASK & P4IN) {Extension_Locate_PIN(6, i, 4, (P4MASK & P4IN)); f=0;}
              if (P5MASK & P5IN) {Extension_Locate_PIN(6, i, 5, (P5MASK & P5IN)); f=0;}
              if (P6MASK & P6IN & ~(1<<i)) {Extension_Locate_PIN(6, i, 6, (P6MASK & P6IN) & ~(1<<i)); f=0;}
              if (P7MASK & P7IN) {Extension_Locate_PIN(6, i, 7, (P7MASK & P7IN)); f=0;}
              if (P8MASK & P8IN) {Extension_Locate_PIN(6, i, 8, (P8MASK & P8IN)); f=0;}
              if (P9MASK & P9IN) {Extension_Locate_PIN(6, i, 9, (P9MASK & P9IN)); f=0;}
              if (P10MASK & P10IN) {Extension_Locate_PIN(6, i, 10, (P10MASK & P10IN)); f=0;}
              if (P11MASK & P11IN) {Extension_Locate_PIN(6, i, 11, (P11MASK & P11IN)); f=0;}
              P6DIR = P6DIR & ~(1<<i);
              P6OUT = P6OUT & ~(1<<i);
            }
          g_f = g_f && f;
          if (f)
            Terminal_Write(" - OK");
          /*else
            return 0;*/
        //Test PORT6 -

            Terminal_NewLine();

        //Test PORT7 +
          f=1;
          Terminal_Write("Test PORT7");
          for (i=0; i<=7; i++)
            if (P7MASK & 1<<i)
            {
              P7DIR = P7DIR | 1<<i;
              P7OUT = P7OUT | 1<<i;
              Delay(HOLD_TIME);
              if (P1MASK & P1IN) {Extension_Locate_PIN(7, i, 1, (P1MASK & P1IN)); f=0;}
              if (P2MASK & P2IN) {Extension_Locate_PIN(7, i, 2, (P2MASK & P2IN)); f=0;}
              if (P3MASK & P3IN) {Extension_Locate_PIN(7, i, 3, (P3MASK & P3IN)); f=0;}
              if (P4MASK & P4IN) {Extension_Locate_PIN(7, i, 4, (P4MASK & P4IN)); f=0;}
              if (P5MASK & P5IN) {Extension_Locate_PIN(7, i, 5, (P5MASK & P5IN)); f=0;}
              if (P6MASK & P6IN) {Extension_Locate_PIN(7, i, 6, (P6MASK & P6IN)); f=0;}
              if (P7MASK & P7IN & ~(1<<i)) {Extension_Locate_PIN(7, i, 7, (P7MASK & P7IN) & ~(1<<i)); f=0;}
              if (P8MASK & P8IN) {Extension_Locate_PIN(7, i, 8, (P8MASK & P8IN)); f=0;}
              if (P9MASK & P9IN) {Extension_Locate_PIN(7, i, 9, (P9MASK & P9IN)); f=0;}
              if (P10MASK & P10IN) {Extension_Locate_PIN(7, i, 10, (P10MASK & P10IN)); f=0;}
              if (P11MASK & P11IN) {Extension_Locate_PIN(7, i, 11, (P11MASK & P11IN)); f=0;}
              P7DIR = P7DIR & ~(1<<i);
              P7OUT = P7OUT & ~(1<<i);
            }
          g_f = g_f && f;
          if (f)
            Terminal_Write(" - OK");
          /*else
            return 0;*/
        //Test PORT7 -

            Terminal_NewLine();

        //Test PORT8 +
          f=1;
          Terminal_Write("Test PORT8");
          for (i=0; i<=7; i++)
            if (P8MASK & 1<<i)
            {
              P8DIR = P8DIR | 1<<i;
              P8OUT = P8OUT | 1<<i;
              Delay(HOLD_TIME);
              if (P1MASK & P1IN) {Extension_Locate_PIN(8, i, 1, (P1MASK & P1IN)); f=0;}
              if (P2MASK & P2IN) {Extension_Locate_PIN(8, i, 2, (P2MASK & P2IN)); f=0;}
              if (P3MASK & P3IN) {Extension_Locate_PIN(8, i, 3, (P3MASK & P3IN)); f=0;}
              if (P4MASK & P4IN) {Extension_Locate_PIN(8, i, 4, (P4MASK & P4IN)); f=0;}
              if (P5MASK & P5IN) {Extension_Locate_PIN(8, i, 5, (P5MASK & P5IN)); f=0;}
              if (P6MASK & P6IN) {Extension_Locate_PIN(8, i, 6, (P6MASK & P6IN)); f=0;}
              if (P7MASK & P7IN) {Extension_Locate_PIN(8, i, 7, (P7MASK & P7IN)); f=0;}
              if (P8MASK & P8IN & ~(1<<i)) {Extension_Locate_PIN(8, i, 8, (P8MASK & P8IN) & ~(1<<i)); f=0;}
              if (P9MASK & P9IN) {Extension_Locate_PIN(8, i, 9, (P9MASK & P9IN)); f=0;}
              if (P10MASK & P10IN) {Extension_Locate_PIN(8, i, 10, (P10MASK & P10IN)); f=0;}
              if (P11MASK & P11IN) {Extension_Locate_PIN(8, i, 11, (P11MASK & P11IN)); f=0;}
              P8DIR = P8DIR & ~(1<<i);
              P8OUT = P8OUT & ~(1<<i);
            }
          g_f = g_f && f;
          if (f)
            Terminal_Write(" - OK");
          /*else
            return 0;*/
        //Test PORT8 -

            Terminal_NewLine();

        //Test PORT9 +
          f=1;
          Terminal_Write("Test PORT9");
          for (i=0; i<=7; i++)
            if (P9MASK & 1<<i)
            {
              P9DIR = P9DIR | 1<<i;
              P9OUT = P9OUT | 1<<i;
              Delay(HOLD_TIME);
              if (P1MASK & P1IN) {Extension_Locate_PIN(9, i, 1, (P1MASK & P1IN)); f=0;}
              if (P2MASK & P2IN) {Extension_Locate_PIN(9, i, 2, (P2MASK & P2IN)); f=0;}
              if (P3MASK & P3IN) {Extension_Locate_PIN(9, i, 3, (P3MASK & P3IN)); f=0;}
              if (P4MASK & P4IN) {Extension_Locate_PIN(9, i, 4, (P4MASK & P4IN)); f=0;}
              if (P5MASK & P5IN) {Extension_Locate_PIN(9, i, 5, (P5MASK & P5IN)); f=0;}
              if (P6MASK & P6IN) {Extension_Locate_PIN(9, i, 6, (P6MASK & P6IN)); f=0;}
              if (P7MASK & P7IN) {Extension_Locate_PIN(9, i, 7, (P7MASK & P7IN)); f=0;}
              if (P8MASK & P8IN) {Extension_Locate_PIN(9, i, 8, (P8MASK & P8IN)); f=0;}
              if (P9MASK & P9IN & ~(1<<i)) {Extension_Locate_PIN(9, i, 9, (P9MASK & P9IN) & ~(1<<i)); f=0;}
              if (P10MASK & P10IN) {Extension_Locate_PIN(9, i, 10, (P10MASK & P10IN)); f=0;}
              if (P11MASK & P11IN) {Extension_Locate_PIN(9, i, 11, (P11MASK & P11IN)); f=0;}
              P9DIR = P9DIR & ~(1<<i);
              P9OUT = P9OUT & ~(1<<i);
            }
          g_f = g_f && f;
          if (f)
            Terminal_Write(" - OK");
          /*else
            return 0;*/
        //Test PORT9 -

            Terminal_NewLine();

        //Test PORT10 +
          f=1;
          Terminal_Write("Test PORT10");
          for (i=0; i<=7; i++)
            if (P10MASK & 1<<i)
            {
              P10DIR = P10DIR | 1<<i;
              P10OUT = P10OUT | 1<<i;
              Delay(HOLD_TIME);
              if (P1MASK & P1IN) {Extension_Locate_PIN(10, i, 1, (P1MASK & P1IN)); f=0;}
              if (P2MASK & P2IN) {Extension_Locate_PIN(10, i, 2, (P2MASK & P2IN)); f=0;}
              if (P3MASK & P3IN) {Extension_Locate_PIN(10, i, 3, (P3MASK & P3IN)); f=0;}
              if (P4MASK & P4IN) {Extension_Locate_PIN(10, i, 4, (P4MASK & P4IN)); f=0;}
              if (P5MASK & P5IN) {Extension_Locate_PIN(10, i, 5, (P5MASK & P5IN)); f=0;}
              if (P6MASK & P6IN) {Extension_Locate_PIN(10, i, 6, (P6MASK & P6IN)); f=0;}
              if (P7MASK & P7IN) {Extension_Locate_PIN(10, i, 7, (P7MASK & P7IN)); f=0;}
              if (P8MASK & P8IN) {Extension_Locate_PIN(10, i, 8, (P8MASK & P8IN)); f=0;}
              if (P9MASK & P9IN) {Extension_Locate_PIN(10, i, 9, (P9MASK & P9IN)); f=0;}
              if (P10MASK & P10IN & ~(1<<i)) {Extension_Locate_PIN(10, i, 10, (P10MASK & P10IN) & ~(1<<i)); f=0;}
              if (P11MASK & P11IN) {Extension_Locate_PIN(10, i, 11, (P11MASK & P11IN)); f=0;}
              P10DIR = P10DIR & ~(1<<i);
              P10OUT = P10OUT & ~(1<<i);
            }
          g_f = g_f && f;
          if (f)
            Terminal_Write(" - OK");
          /*else
            return 0;*/
        //Test PORT10 -

            Terminal_NewLine();

        //Test PORT11 +
          f=1;
          Terminal_Write("Test PORT11");
          for (i=0; i<=7; i++)
            if (P11MASK & 1<<i)
            {
              P11DIR = P11DIR | 1<<i;
              P11OUT = P11OUT | 1<<i;
              Delay(HOLD_TIME);
              if (P1MASK & P1IN) {Extension_Locate_PIN(11, i, 1, (P1MASK & P1IN)); f=0;}
              if (P2MASK & P2IN) {Extension_Locate_PIN(11, i, 2, (P2MASK & P2IN)); f=0;}
              if (P3MASK & P3IN) {Extension_Locate_PIN(11, i, 3, (P3MASK & P3IN)); f=0;}
              if (P4MASK & P4IN) {Extension_Locate_PIN(11, i, 4, (P4MASK & P4IN)); f=0;}
              if (P5MASK & P5IN) {Extension_Locate_PIN(11, i, 5, (P5MASK & P5IN)); f=0;}
              if (P6MASK & P6IN) {Extension_Locate_PIN(11, i, 6, (P6MASK & P6IN)); f=0;}
              if (P7MASK & P7IN) {Extension_Locate_PIN(11, i, 7, (P7MASK & P7IN)); f=0;}
              if (P8MASK & P8IN) {Extension_Locate_PIN(11, i, 8, (P8MASK & P8IN)); f=0;}
              if (P9MASK & P9IN) {Extension_Locate_PIN(11, i, 9, (P9MASK & P9IN)); f=0;}
              if (P10MASK & P10IN) {Extension_Locate_PIN(11, i, 10, (P10MASK & P10IN)); f=0;}
              if (P11MASK & P11IN & ~(1<<i)) {Extension_Locate_PIN(11, i, 11, (P11MASK & P11IN) & ~(1<<i)); f=0;}
              P11DIR = P11DIR & ~(1<<i);
              P11OUT = P11OUT & ~(1<<i);
            }
          g_f = g_f && f;
          if (f)
            Terminal_Write(" - OK");
          /*else
            return 0;*/
        //Test PORT11 -

      //Test PIN to PIN -

        return 0;
    }

  //Extension functions -

  //UExt functions +
    void UExt_Init()
    {
      P3DIR = P3DIR & 0x0F;     //P3.4 - P3.7 set as input
      P3SEL = P3SEL & 0x0F;     //P3.4 - P3.7 set as alternate functions

      P5DIR = P5DIR & 0x0F;     //P5.4 - P5.7 set as input
      P5SEL = P5SEL & 0x0F;     //P5.4 - P5.7 set as alternate functions
    }

    unsigned char UExt_Check()
    {
      int i;

      //Test to GND +

      Terminal_Write("Test GND");
      f=1;
      P3DIR = P3DIR | 0x40;     //P3.6 (Pullpin) set as output
      P3OUT = P3OUT | 0x40;     //P3.6 = 1
      Delay(HOLD_TIME);
      if (UMASK & ~P3IN) {Extension_Locate(UMASK & ~P3IN, 3); f=0;}
      if (UMASK & ~P5IN) {Extension_Locate(UMASK & ~P5IN, 5); f=0;}
      g_f = g_f && f;
      if (f)
        Terminal_WriteLn(" - OK");
      else
        Terminal_NewLine();

      //Test to GND -

      Terminal_NewLine();

      //Test PIN to VCC +

      Terminal_Write("Test VCC");
      f=1;
      P3DIR = P3DIR | 0x40;     //P3.6 (Pullpin) set as output
      P3OUT = P3OUT & 0xBF;     //P3.6 = 0
      Delay(HOLD_TIME);
      if (UMASK & P3IN) {Extension_Locate(UMASK & P3IN, 3); f=0;}
      if (UMASK & P5IN) {Extension_Locate(UMASK & P5IN, 5); f=0;}
      g_f = g_f && f;
      if (f)
        Terminal_WriteLn(" - OK");
      else
        Terminal_NewLine();

      //Test PIN to VCC -

      Terminal_NewLine();

      //Test PIN to PullPin +

        Terminal_Write("Test PullPin (P3.6 - 41)");
        f=1;
	//Running 0 +
        P3OUT = P3OUT | 0x40;     //P3.6 (PullPIN) = 1
        P3DIR = P3DIR | 0x40;     //P3.6 (Pullpin) set as output

        //P3 to PullPin +
        for (i=4; i<=7; i++)
        {
          test_const = 1<<i;
          if (i!=6)
          {
	      P3OUT = P3OUT & ~(test_const);  //P3.i (TestPIN) = 0
	      P3DIR = P3DIR | test_const;     //P3.i (TestPIN) set as output
              for (counter=TIMEOUT; counter; counter--)        //counter used as timeout
              {
                Delay(PULL_HOLD_TIME);
                if (P3IN & test_const)
		{
                  Extension_Locate_PullPin(3, i);
		  break;
		}
	      }
	      P3DIR = P3DIR & ~(test_const);
	  }
	}
        //P3 to PullPin -

        //P5 to PullPin +
        for (i=4; i<=7; i++)
        {
          test_const = 1<<i;
	  P5OUT = P5OUT & ~(test_const);  //P5.i (TestPIN) = 0
	  P5DIR = P5DIR | test_const;     //P5.i (TestPIN) set as output
          for (counter=TIMEOUT; counter; counter--)        //counter used as timeout
          {
            Delay(PULL_HOLD_TIME);
            if (P5IN & test_const)
	    {
              Extension_Locate_PullPin(5, i);
	      break;
	    }
	  }
	  P5DIR = P5DIR & ~(test_const);
	}
        //P5 to PullPin -

        g_f = g_f && f;
        if (f)
          Terminal_WriteLn(" - OK");
        else
          Terminal_NewLine();
      //Test PIN to PullPin -

      if (!g_f) return 0;
      Terminal_NewLine();

      //Test PIN to PIN +
        P3OUT = P3OUT & ~(0x40);    //P3.6 (PullPIN) = 0
        Terminal_WriteLn("Test PIN to PIN:");

        //Test Port3 +
          f=1;
          Terminal_Write("Test PORT3");
          for (i=0; i<=7; i++)
            if ((i!=6) && (UMASK & 1<<i))
            {
              P3DIR = P3DIR | 1<<i;
              P3OUT = P3OUT | 1<<i;
              Delay(HOLD_TIME);
              if (UMASK & P3IN & ~(1<<i)) {Extension_Locate_PIN(3, i, 3, (UMASK & P3IN) & ~(1<<i)); f=0;}
              if (UMASK & P5IN) {Extension_Locate_PIN(3, i, 5, (UMASK & P5IN)); f=0;}
              P3DIR = P3DIR & ~(1<<i);
              P3OUT = P3OUT & ~(1<<i);
            }
          g_f = g_f && f;
          if (f)
            Terminal_Write(" - OK");
        //Test Port3 -

          Terminal_NewLine();

        //Test Port5 +
          f=1;
          Terminal_Write("Test PORT5");
          for (i=0; i<=7; i++)
            if (UMASK & 1<<i)
            {
              P5DIR = P5DIR | 1<<i;
              P5OUT = P5OUT | 1<<i;
              Delay(HOLD_TIME);
              if (UMASK & P3IN) {Extension_Locate_PIN(5, i, 3, (UMASK & P3IN)); f=0;}
              if (UMASK & P5IN & ~(1<<i)) {Extension_Locate_PIN(5, i, 5, (UMASK & P5IN) & ~(1<<i)); f=0;}
              P5DIR = P5DIR & ~(1<<i);
              P5OUT = P5OUT & ~(1<<i);
            }
          g_f = g_f && f;
          if (f)
            Terminal_Write(" - OK");
        //Test Port5 -

      //Test PIN to PIN -

        Terminal_NewLine();

      return 0;
    }
  //UExt functions -

  //TEST, MENU, CONTROL functions +

    void Joystick_Check(char menu)
    {
      char max_line, max_page;
      switch (menu)
      {
        case 1:
		if (page==1)
            max_line=5;
		else
	  	  max_line=3;
          max_page=2;
        break;
        case 11:
          max_line=4;
          max_page=1;
        break;
        case 12:
          max_line=4;
          max_page=1;
        break;
        case 13:
          max_line=1;
          max_page=1;
        break;
      }


        if ((~P1IN & 0x04) && released)           //1<<2
        {
          released = 0;
          line++;
          if (line>max_line) line=1;
        }

        if ((~P1IN & 0x08) && released)           //1<<3
        {
          released = 0;
          line--;
          if (line==0) line=max_line;
        }

        if ((~P1IN & 0x20) && released)           //1<<5
        {
          released = 0;
          line=1;
          page++;
          if (page>max_page) page=1;
        }

        if ((~P1IN & 0x40) && released)           //1<<6
        {
          released = 0;
          line=1;
          page--;
          if (page==0) page=max_page;
        }

      //middle button +
        if ((~P1IN & 0x10) && released)           //1<<4
        {
          released = 0;
          switch (menu)
          {
            case 1:
              Action(page, line);
            break;
            case 11:
              switch (line)
              {
                case 1:
                  P10OUT = P10OUT | 0x80;       //1<<7
                break;
                case 2:
                  P10OUT = P10OUT & 0x7F;       //~(1<<7)
                break;
                case 3:
                  for (counter = 3; counter; counter--)
                  {
                    P10OUT = P10OUT | 0x80; Delay(10000);       //1<<7
                    P10OUT = P10OUT & 0x7F; Delay(10000);       //~(1<<7)
                  }
                break;
                case 4:
                  break_f = 1;
                  P10OUT = P10OUT & 0x7F;       //~(1<<7)
                break;
              }
            break;
            case 12:
              switch (line)
              {
                case 1:
                  P10OUT = P10OUT | 0x40;       //1<<6
                break;
                case 2:
                  P10OUT = P10OUT & 0xBF;       //~(1<<6)
                break;
                case 3:
                  for (counter = 3; counter; counter--)
                  {
                    P10OUT = P10OUT | 0x40; Delay(10000);     //1<<6
                    P10OUT = P10OUT & 0xBF; Delay(10000);     //~(1<<6)
                  }
                break;
                case 4:
                  break_f = 1;
                  P10OUT = P10OUT & 0xBF;      //~(1<<6)
                break;
              }
            break;
          }
        }
      //middle button -

      //check for release the buttons
      if ((P1IN & 0x7C) == 0x7C)
        released = 1;
    }

    void Action(char p, char l)
    {
      switch (p)
      {
        case 1:
          switch(l)
          {
            case 1:
              TEST_LED_1();
            break;
            case 2:
              TEST_LED_2();
            break;
            case 3:
              TEST_ADC();
            break;
            case 4:
              TEST_Extension();
            break;
            case 5:
              TEST_UExt();
            break;
          }
        break;

        case 2:
          switch(l)
          {
            case 1:
	      LCD_Inverse();
            break;
            case 2:
    	         Test_MMC();	//Penko's software
              LCD_WriteLn("OK");
	         LCD_Update(MMC_Flag, 1<<2);
      	    while (!((P1IN & 0x7C) == 0x7C));         //wait until buttons are released
      	    while (!(~P1IN & 0x10));                  //wait until center button is pressed
			line = 1;
            break;
            case 3:
			Test_UART3();
            break;
            case 4:
            break;
            case 5:
            break;
          }
        break;
      }
    }

    void TEST_LED_1()
    {
      line = 1;
      while (1)
      {
        LCD_Menu(11);
        Joystick_Check(11);
        if (break_f)
        {
          line = 1;
          break_f = 0;
          break;
        }
      }
    }

    void TEST_LED_2()
    {
      line = 1;
      while (1)
      {
        LCD_Menu(12);
        Joystick_Check(12);
        if (break_f)
        {
          line = 1;
          break_f = 0;
          break;
        }
      }
    }

    void TEST_ADC()
    {
      line = 1;
      LCD_Menu(13);
      while (!((P1IN & 0x7C) == 0x7C));         //wait until buttons are released
      TA0CCR0 = 0xFF;
      while (1)
      {
        TA0CCR1 = ADC12_Receive();
        if (~P1IN & 0x10)
        {
          line = 1;
          released = 0;
          break;
        }
      }
      TA0CCR0 = 0x00;
    }

    void TEST_Extension()
    {
      LCD_Clear();
      LCD_WriteLn("Please wait");
      LCD_Update(1, 0);

      g_f = 1;
      Extension_Init();

      Terminal_NewLine();
      Terminal_WriteLn("-------Extensions TEST-------");
      Extension_Check();
      Terminal_NewLine();
      Terminal_WriteLn("-----------------------------");

      PORT_Init();          //reinitialize PORTS after extension test

      LCD_Clear();
      LCD_WriteLn("Test result:");
      if (g_f)
      {
        Terminal_WriteLn("Extension Test - OK");
        LCD_WriteLn("TEST OK!");
        LCD_WriteLn("OK");
        LCD_Update(1, 1<<2);
      }
      else
      {
        Terminal_WriteLn("Extension Test - Failed");
        LCD_WriteLn("TEST FAILED!");
        LCD_WriteLn("Check terminal for details");
        LCD_WriteLn("OK");
        LCD_Update(0, 1<<4);
      }

      while (!((P1IN & 0x7C) == 0x7C));         //wait until buttons are released
      while (!(~P1IN & 0x10));                  //wait until center button is pressed
      line = 1;
    }

    void TEST_UExt()
    {
      LCD_Clear();
      LCD_WriteLn("Please wait");
      LCD_Update(1, 0);

      g_f = 1;
      UExt_Init();

      Terminal_NewLine();
      Terminal_WriteLn("-----------UExt TEST---------");
      UExt_Check();
      Terminal_NewLine();
      Terminal_WriteLn("-----------------------------");


      PORT_Init();          //reinitialize PORTS after UExt test

      LCD_Clear();
      LCD_WriteLn("Test result:");
      if (g_f)
      {
        Terminal_WriteLn("UExt Test - OK");
        LCD_WriteLn("TEST OK!");
        LCD_WriteLn("OK");
        LCD_Update(1, 1<<2);
      }
      else
      {
        Terminal_WriteLn("UExt Test - Failed");
        LCD_WriteLn("TEST FAILED!");
        LCD_WriteLn("Check terminal for details");
        LCD_WriteLn("OK");
        LCD_Update(0, 1<<4);
      }

      while (!((P1IN & 0x7C) == 0x7C));         //wait until buttons are released
      while (!(~P1IN & 0x10));                  //wait until center button is pressed
      line = 1;
    }

    void Test_UART3()
    {
	 char sign;
	 LCD_Menu(23);
	 UART3_Init(BAUDRATE);
      while (!((P1IN & 0x7C) == 0x7C));         //wait until buttons are released
      while (!(~P1IN & 0x10))                   //wait until center button is pressed
	 {
                if (UCA2IFG & 0x01) UART2_Transmit(UCA2RXBUF+1); //test UART2 echo
		sign = UART3_Receive();
		if (sign)
			UART3_Transmit(sign+1);
	 }
	 line=1;
    }


  //TEST, MENU, CONTROL functions -

//definitions of functions -


//MAIN functions

int main(void)
{
  // Stop watchdog timer to prevent time out reset
  WDTCTL = WDTPW + WDTHOLD;
  //__bis_SR_register(GIE);

  //Initialize +
    PORT_Init();
    PIN_Init();
    CLOCK_XT1_Init();
    CLOCK_XT2_Init();
    UART2_Init(BAUDRATE);
    UART3_Init(BAUDRATE);
    SPI2_Init(LCD_FREQ);
    SPI0_Init(MMC_FREQ);
    LCD_Init();
    ADC12_Init();
    TIMER_A_Init();
  //Initialize -

  //Endless loop
  while (1)
  {
    LCD_Menu(page);
    Joystick_Check(1);
  }
}
