/**********************************************************************************/
/*    Test program for:															  */
/*	  Board: AVR-PX128A1														  */
/*    Manufacture: OLIMEX                                                   	  */
/*	  COPYRIGHT (C) 2008														  */
/*    Designed by: Engineer Penko T. Bozhkov                                      */
/*    Module Name    :  MultiMediaCard in SPI mode                                */
/*    File   Name    :  mmc.h                                                     */
/*    Revision       :  Rev.B                                                     */
/*    Date           :  20.03.2009					                              */
/**********************************************************************************/
#ifndef __MMC_H
#define __MMC_H

/***** 1.All functions prototypes *****/
unsigned char mmcSendByte(unsigned char spiDataByte);
void mmcSendCmd(const char cmd, unsigned long data, const char crc);
char mmcGetResponse(void);
char initMMC(void);
char mmcSetBlockLength(const unsigned int blocklength);
char mmcReadRegister(const char cmd_register, const unsigned char length_in_bytes);
char mmcWriteBlock(const unsigned long address);
//char mmcReadBlock(const unsigned long address, const unsigned int count);
//char mmcRead_WriteVerification(void);
char mmcRead_VerifyBlock(const unsigned long address, const unsigned int count);
char mmcCheckBusy(void);
char mmcCapacity(void);


/***** 2.All definitions *****/
#define MMC_INIT_SUCCESS           				0x00
#define MMC_INIT_FAIL							0x01
#define	MMC_SetBL_SUCCESS						0x02
#define	MMC_SetBL_ERROR							0x03
#define	MMC_ReadREG_SUCCESS						0x04
#define MMC_RESPONSE_ERROR     					0x05
#define MMC_BLOCK_SET_ERROR						0x06
#define MMC_CRC_ERROR         					0x07
#define MMC_WRITE_ERROR       					0x08
#define MMC_OTHER_ERROR       					0x09
#define MMC_WRITE_SUCCESS						0x0A
#define	MMC_READ_SUCCESS						0x0B
#define	MMC_READ_DATA_TOKEN_ERROR				0x0C
#define	MMC_READ_RESPONSE_ERROR					0x0D
#define	MMC_READ_BLOCK_SET_ERROR				0x0E
#define	MMC_Read_Write_Verification_Fail		0x0F
#define	MMC_Read_Write_Verification_Success		0x10
#define mmcMemory_less_than_64MB				0x20
#define mmcMemory_64MB							0x21
#define mmcMemory_128MB							0x22
#define mmcMemory_256MB							0x23
#define mmcMemory_512MB							0x24
#define mmcMemory_1GB							0x25
#define mmcMemory_2GB							0x26
#define mmcMemory_4GB							0x27
#define	mmcMemory_more_than_4GB					0x28
#define	mmcMemory_Calculation_Error				0x29

#define MMC_START_DATA_BLOCK_TOKEN 				0xFE	// Data token start byte, Start Single Block Read

//#define	CP_E							0x00	// When Card Present signal is not used set CP_E = 0x00
//#define	WP_E							0x00	// When Write Protect signal is not used set WP_E = 0x00

//#define CP_check					PORTQ.IN & 0x08
//#define	WP_Check					PORTQ.IN & 0x04
#define MMC_NSS_Low				P3OUT &= ~0x01;	P3DIR |= 0x01;
#define MMC_NSS_High				P3OUT |= 0x01;  P3DIR |= 0x01;


/***** 3.All variables *****/
extern volatile unsigned char mmc_write_buffer[512];
//extern volatile unsigned char mmc_read_buffer[512];
extern volatile unsigned char mmcReg_buffer[16];

#endif //__MMC_H
